from datetime import datetime

import pandas as pd


class PlayTypes:

    PERMODE = {
        "PERGAME": "PerGame",
        "PER36": "Per36",
        "TOTALS": "Totals",
        "PERMINUTE": "PerMinute",
        "PERMIN": "PerMinute",
        "PERPOSSESSION": "Per100Possessions",
        "PERPOSS": "Per100Possessions",
        "PER100POSSESSIONS": "Per100Possessions",
        "PERPLAY": "PerPlay",
        "PER100PLAYS": "Per100Plays",
    }

    PLAYTYPES = {
        "TRANSITION": "Transition",
        "ISOLATION": "Isolation",
        "ISO": "Isolation",
        "PRBALLHANDLER": "PRBallHandler",
        "PRROLLMAN": "PRRollman",
        "POSTUP": "Postup",
        "SPOTUP": "Spotup",
        "HANDOFF": "Handoff",
        "CUT": "Cut",
        "OFFSCREEN": "OffScreen",
        "PUTBACKS": "OffRebound",
        "OFFREBOUND": "OffRebound",
        "MISC": "Misc",
    }

    TRACKING_TYPES = {
        "SPEEDDISTANCE": "SpeedDistance",
        "SPEED": "SpeedDistance",
        "DISTANCE": "SpeedDistance",
        "POSSESSIONS": "Possessions",
        "CATCHSHOOT": "CatchShoot",
        "PULLUPSHOT": "PullUpShot",
        "PULLUP": "PullUpShot",
        "DEFENSE": "Defense",
        "DRIVES": "Drives",
        "DRIVE": "Drives",
        "PASSING": "Passing",
        "ELBOWTOUCH": "ElbowTouch",
        "ELBOW": "ElbowTouch",
        "POSTTOUCH": "PostTouch",
        "POST": "PostTouch",
        "PAINTTOUCH": "PaintTouch",
        "PAINT": "PaintTouch",
        "EFFICIENCY": "Efficiency",
    }

    DEFENSE_TYPES = {
        "OVERALL": "Overall",
        "THREE": "3 Pointers",
        "THREES": "3 Pointers",
        "TWOS": "2 Pointers",
        "TWO": "2 Pointers",
        "LESSTHAN6FT": "Less Than 6Ft",
        "LESSTHAN10FT": "Less Than 10Ft",
        "GREATERTHAN15FT": "Greater Than 15Ft",
    }


class Formatter:

    def get_current_season_year() -> str:
        current_datetime = datetime.now()
        current_season_year = current_datetime.year
        if current_datetime.month <= 9:
            current_season_year -= 1
        return current_season_year

    def format_season(season_year: int) -> str:
        return "{}-{}".format(int(season_year), str(int(season_year) + 1)[2:])

    def format_game_id(game_id) -> str:
        return str(game_id).zfill(10)

    def combine_strings(row) -> str:
        return next(value for value in row if pd.notna(value))

    def check_playtype(play: str, playtypes: dict) -> str:

        play = play.replace("_", "").replace("-", "").upper()

        if play == "ALL":
            return list(set(playtypes.values()))

        if play not in set(playtypes.keys()):
            raise ValueError(f"Playtype: {play} not found")

        return playtypes[play]



from io import BytesIO

import nba_api.stats.endpoints as nba
import pandas as pd
import requests
from bs4 import BeautifulSoup
from loguru import logger
from nba_api.stats.endpoints import leaguegamefinder
from nba_api.stats.static import players, teams
from PIL import Image

from nbastatpy.utils import Formatter, PlayTypes


class Player:
    def __init__(
        self,
        player: str,
        season_year: str = None,
        playoffs: bool = False,
        permode: str = "PERGAME",
    ):
        """
        Initializes a Player object.

        Args:
            player (str): The name or ID of the player.
            season_year (str, optional): The season year. Defaults to None.
            playoffs (bool, optional): Whether to retrieve playoff data. Defaults to False.
            permode (str, optional): The per mode for the player's stats. Defaults to "PERGAME".
        """
        self.permode = PlayTypes.PERMODE[
            permode.replace("_", "").replace("-", "").upper()
        ]
        self.name_meta = players.find_player_by_id(player)
        if self.name_meta:
            self.name_meta = [self.name_meta]
        else:
            self.name_meta = players.find_players_by_full_name(player)

        if not self.name_meta:
            raise ValueError(f"{player} not found")
        if len(self.name_meta) > 1:
            logger.warning(
                f"Multiple players returned. Using: {self.name_meta[0]['full_name']}"
            )
        self.id = self.name_meta[0]["id"]
        self.name = self.name_meta[0]["full_name"]
        self.first_name = self.name_meta[0]["first_name"]
        self.last_name = self.name_meta[0]["last_name"]
        self.is_active = self.name_meta[0]["is_active"]

        if season_year:
            self.season_year = season_year
        else:
            self.season_year = Formatter.get_current_season_year()
        self.season = Formatter.format_season(self.season_year)
        self.season_type = "Regular Season"
        if playoffs:
            self.season_type = "Playoffs"


    def get_common_info(self) -> pd.DataFrame:
        """Gets common info like height, weight, draft_year, etc. and sets as class attr
        
        Returns:
            pd.DataFrame: A DataFrame containing the common information of the player.
        """
        self.common_info = (
            nba.CommonPlayerInfo(self.id).get_data_frames()[0].iloc[0].to_dict()
        )

        for attr_name, value in self.common_info.items():
            setattr(self, attr_name.lower(), self.common_info.get(attr_name, None))

        return self.common_info

    def get_salary(self) -> pd.DataFrame:
        """
        Retrieves the salary information for a player from hoopshype.com.

        Returns:
            pd.DataFrame: A DataFrame containing the salary information for the player.
        """
        salary_url = f"https://hoopshype.com/player/{self.first_name}-{self.last_name}/salary/".lower()
        result = requests.get(salary_url)
        soup = BeautifulSoup(result.content, features="html.parser")
        tables = soup.find_all("table")
        if len(tables) > 1:
            # Get the table rows
            rows = [[cell.text.strip() for cell in row.find_all('td')] for row in tables[0].find_all('tr')]

            rows2 = [[cell.text.strip() for cell in row.find_all('td')] for row in tables[1].find_all('tr')]
            
            projected = pd.DataFrame(rows[1:], columns=rows[0])
            projected["Team"] = projected.columns[1]
            projected = projected.rename(columns={projected.columns[1]:"Salary"})
            projected["Salary_Type"] = "Projected"
            
            historical = pd.DataFrame(rows2[1:], columns=rows2[0])
            historical["Salary_Type"] = "Historical"
            
            self.salary_df = pd.concat([projected, historical])
            
        else:
            # Get the table rows
            rows = [[cell.text.strip() for cell in row.find_all('td')] for row in tables[0].find_all('tr')]
            self.salary_df = pd.DataFrame(rows[1:], columns=rows[0])

        return self.salary_df

    def get_headshot(self):
        """
        Retrieves the headshot image of the player.

        Returns:
            PIL.Image.Image: The headshot image of the player.
        """
        pic_url = f"https://cdn.nba.com/headshots/nba/latest/1040x760/{self.id}.png"
        pic = requests.get(pic_url)
        self.headshot = Image.open(BytesIO(pic.content))
        return self.headshot

    def get_season_career_totals(self) -> pd.DataFrame:
        """Gets seasons and career data

        Returns:
            pd.DataFrame: 2 dataframes, season totals and career
        """
        df_list = nba.PlayerCareerStats(player_id=self.id).get_data_frames()
        self.career_totals = df_list[1]
        self.season_totals = df_list[0]
        return self.season_totals, self.career_totals

    def get_splits(self) -> pd.DataFrame:
        """Gets all splits for a given season"""

        self.splits_data = pd.concat(
            nba.PlayerDashboardByGeneralSplits(
                self.id,
                season=self.season,
                season_type_playoffs=self.season_type,
                per_mode_detailed=self.permode,
            ).get_data_frames()
        )

        return self.splits_data

    def get_game_splits(self) -> pd.DataFrame:
        """Gets splits for the game (halftime, quarter, etc.)

        Returns:
            pd.DataFrame: dataframe with all game splits for a season
        """

        self.game_splits = pd.concat(
            nba.PlayerDashboardByGameSplits(
                self.id,
                season=self.season,
                season_type_playoffs=self.season_type,
                per_mode_detailed=self.permode,
            ).get_data_frames()
        )
        return self.game_splits

    def get_shooting_splits(self) -> pd.DataFrame:

        self.shooting_splits = pd.concat(
            nba.PlayerDashboardByShootingSplits(
                self.id,
                season=self.season,
                season_type_playoffs=self.season_type,
                per_mode_detailed=self.permode,
            ).get_data_frames()
        )
        return self.shooting_splits

    def get_combine_stats(self) -> pd.DataFrame:
        """Gets draft combine data for the player's draft year"""
        if not hasattr(
            self, "draft_year"
        ):  # Check if we know the player's draft year yet
            self.get_common_info()

        self.combine_stats = nba.DraftCombineStats(
            season_all_time=self.draft_year
        ).get_data_frames()[0]

        self.combine_nonstationary_shooting = nba.DraftCombineNonStationaryShooting(
            season_year=self.draft_year
        ).get_data_frames()[0]

        self.combine_spot_shooting = nba.DraftCombineSpotShooting(
            season_year=self.draft_year
        ).get_data_frames()[0]

        return [
            self.combine_stats,
            self.combine_nonstationary_shooting,
            self.combine_spot_shooting,
        ]

    def get_awards(self) -> pd.DataFrame:
        """Gets any awards won by the player.

        Returns:
            pd.DataFrame: A DataFrame containing the player's awards.
        """
        self.awards = nba.PlayerAwards(self.id).get_data_frames()[0]
        return self.awards

    def get_games_boxscore(self) -> pd.DataFrame:
        """
        Retrieves the boxscore data for the games played by the player.

        Returns:
            pd.DataFrame: The boxscore data for the player's games.
        """
        self.games_boxscore = leaguegamefinder.LeagueGameFinder(
            player_id_nullable=self.id,
            season_nullable=self.season,
            season_type_nullable=self.season_type,
        ).get_data_frames()[0]
        return self.games_boxscore

    def get_matchups(self, defense: bool = False) -> pd.DataFrame:
        """
        Retrieves the matchups data for the player.

        Args:
            defense (bool, optional): If True, retrieves the defensive matchups data. 
                If False, retrieves the offensive matchups data. Defaults to False.

        Returns:
            pd.DataFrame: The matchups data for the player.
        """
        if defense:
            self.matchups = nba.LeagueSeasonMatchups(
                def_player_id_nullable=self.id,
                season=self.season,
                season_type_playoffs=self.season_type,
                per_mode_simple=self.permode,
            ).get_data_frames()[0]
        else:
            self.matchups = nba.LeagueSeasonMatchups(
                off_player_id_nullable=self.id,
                season=self.season,
                season_type_playoffs=self.season_type,
                per_mode_simple=self.permode,
            ).get_data_frames()[0]
        return self.matchups

    def get_clutch(self) -> pd.DataFrame:
        """Gets clutch data for multiple clutch segments

        Returns:
            pd.DataFrame: dataframe with a given year of clutch segments
        """
        self.clutch = pd.concat(
            nba.PlayerDashboardByClutch(
                player_id=self.id,
                season=self.season,
                season_type_playoffs=self.season_type,
                per_mode_detailed=self.permode,
            ).get_data_frames()
        )
        return self.clutch

    def get_pt_pass(self) -> pd.DataFrame:
        """
        Retrieves the passing statistics for the player.

        Returns:
            pd.DataFrame: A DataFrame containing the passing statistics for the player.
        """
        if not hasattr(self, "season_totals"):
            logger.info("Getting Teams")
            teams = self.get_season_career_totals()[0]
        else:
            teams = self.season_totals.copy()

        teams = teams[(teams["SEASON_ID"] == self.season) & (teams["TEAM_ID"] != 0)][
            "TEAM_ID"
        ].tolist()

        if len(teams) > 1:
            self.pt_pass = []
            for team in teams:
                self.pt_pass.append(
                    pd.concat(
                        nba.PlayerDashPtPass(
                            player_id=self.id,
                            team_id=team,
                            season=self.season,
                            season_type_all_star=self.season_type,
                            per_mode_simple=self.permode,
                        ).get_data_frames()
                    )
                )

            self.pt_pass = pd.concat(self.pt_pass)

        else:
            self.pt_pass = pd.concat(
                nba.PlayerDashPtPass(
                    player_id=self.id,
                    team_id=teams[0],
                    season=self.season,
                    season_type_all_star=self.season_type,
                    per_mode_simple=self.permode,
                ).get_data_frames()
            )

        group_cols = ["PASS_TO", "PASS_FROM"]
        self.pt_pass["GROUP_SET"] = self.pt_pass[group_cols].apply(
            Formatter.combine_strings, axis=1
        )
        self.pt_pass = self.pt_pass.drop(columns=group_cols)

        return self.pt_pass

    def get_pt_reb(self) -> pd.DataFrame:
        """
        Retrieves the rebounds data for the player.

        Returns:
            pd.DataFrame: A DataFrame containing the rebounds data.
        """
        if not hasattr(self, "season_totals"):
            logger.info("Getting Teams")
            teams = self.get_season_career_totals()[0]
        else:
            teams = self.season_totals.copy()

        teams = teams[(teams["SEASON_ID"] == self.season) & (teams["TEAM_ID"] != 0)][
            "TEAM_ID"
        ].tolist()

        if len(teams) > 1:
            self.pt_reb = []
            for team in teams:
                self.pt_reb.append(
                    pd.concat(
                        nba.PlayerDashPtReb(
                            player_id=self.id,
                            team_id=team,
                            season=self.season,
                            season_type_all_star=self.season_type,
                            per_mode_simple=self.permode,
                        ).get_data_frames()
                    )
                )

            self.pt_reb = pd.concat(self.pt_reb)

        else:
            self.pt_reb = pd.concat(
                nba.PlayerDashPtReb(
                    player_id=self.id,
                    team_id=teams[0],
                    season=self.season,
                    season_type_all_star=self.season_type,
                    per_mode_simple=self.permode,
                ).get_data_frames()
            )

        group_cols = [
            "OVERALL",
            "SHOT_TYPE_RANGE",
            "REB_NUM_CONTESTING_RANGE",
            "SHOT_DIST_RANGE",
            "REB_DIST_RANGE",
        ]
        self.pt_reb["GROUP_SET"] = self.pt_reb[group_cols].apply(
            Formatter.combine_strings, axis=1
        )
        self.pt_reb = self.pt_reb.drop(columns=group_cols)

        return self.pt_reb

    def get_defense_against_team(self, opposing_team: str) -> pd.DataFrame:
        """
        Retrieves the defensive statistics of the player against a specific team.

        Args:
            opposing_team (str): The abbreviation of the opposing team.

        Returns:
            pd.DataFrame: A DataFrame containing the defensive statistics of the player against the specified team.
        """
        opp_tm_id = teams.find_team_by_abbreviation(opposing_team)["id"]

        self.defense_against_team = nba.PlayerDashPtShotDefend(
            player_id=self.id,
            team_id=opp_tm_id,
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_simple=self.permode,
        ).get_data_frames()[0]
        return self.defense_against_team

    def get_pt_shots(self) -> pd.DataFrame:
        """
        Retrieves the shots data for the player.

        Returns:
            pd.DataFrame: The shots data for the player.
        """
        if not hasattr(self, "season_totals"):
            logger.info("Getting Teams")
            teams = self.get_season_career_totals()[0]
        else:
            teams = self.season_totals.copy()

        teams = teams[(teams["SEASON_ID"] == self.season) & (teams["TEAM_ID"] != 0)][
            "TEAM_ID"
        ].tolist()

        if len(teams) > 1:
            self.pt_shots = []
            for team in teams:
                self.pt_shots.append(
                    pd.concat(
                        nba.PlayerDashPtShots(
                            player_id=self.id,
                            team_id=team,
                            season=self.season,
                            season_type_all_star=self.season_type,
                            per_mode_simple=self.permode,
                        ).get_data_frames()
                    )
                )

            self.pt_shots = pd.concat(self.pt_shots)

        else:
            self.pt_shots = pd.concat(
                nba.PlayerDashPtShots(
                    player_id=self.id,
                    team_id=teams[0],
                    season=self.season,
                    season_type_all_star=self.season_type,
                    per_mode_simple=self.permode,
                ).get_data_frames()
            )

        group_cols = [
            "SHOT_TYPE",
            "SHOT_CLOCK_RANGE",
            "DRIBBLE_RANGE",
            "CLOSE_DEF_DIST_RANGE",
            "TOUCH_TIME_RANGE",
        ]
        self.pt_shots["GROUP_SET"] = self.pt_shots[group_cols].apply(
            Formatter.combine_strings, axis=1
        )
        self.pt_shots = self.pt_shots.drop(columns=group_cols)

        return self.pt_shots

    def get_shot_chart(self) -> pd.DataFrame:
        """
        Retrieves the shot chart data for the player.

        Returns:
            pd.DataFrame: The shot chart data for the player.
        """
        if not hasattr(self, "season_totals"):
            logger.info("Getting Teams")
            teams = self.get_season_career_totals()[0]
        else:
            teams = self.season_totals.copy()

        teams = teams[(teams["SEASON_ID"] == self.season) & (teams["TEAM_ID"] != 0)][
            "TEAM_ID"
        ].tolist()

        if len(teams) > 1:
            self.shot_chart = []
            for team in teams:
                self.shot_chart.append(
                    nba.ShotChartDetail(
                        player_id=self.id,
                        team_id=team,
                        season_nullable=self.season,
                        season_type_all_star=self.season_type,
                    ).get_data_frames()[0]
                )

            self.shot_chart = pd.concat(self.shot_chart)

        else:
            self.shot_chart = nba.ShotChartDetail(
                player_id=self.id,
                team_id=teams[0],
                season_nullable=self.season,
                season_type_all_star=self.season_type,
            ).get_data_frames()[0]

        return self.shot_chart


if __name__ == "__main__":
    player_name = "giannis"
    player_seas = "2020"
    player = Player(player_name, player_seas)
    print(player.get_salary())



from time import sleep

import nba_api.stats.endpoints as nba
import pandas as pd
import requests
from bs4 import BeautifulSoup
from rich.progress import track

from nbastatpy.utils import Formatter, PlayTypes


class Season:
    def __init__(
        self, season_year: str = None, playoffs=False, permode: str = "PERGAME"
    ):
        """
        Initialize a Season object.

        Args:
            season_year (str, optional): The year of the season. Defaults to None.
            playoffs (bool, optional): Indicates if the season is for playoffs. Defaults to False.
            permode (str, optional): The per mode for the season. Defaults to "PERGAME".
        """
        self.permode = PlayTypes.PERMODE[
            permode.replace("_", "").replace("-", "").upper()
        ]
        if season_year:
            self.season_year = season_year
        else:
            self.season_year = Formatter.get_current_season_year()

        self.season = Formatter.format_season(self.season_year)
        self.season_type = "Regular Season"
        if playoffs:
            self.season_type = "Playoffs"

    def get_salaries(self) -> pd.DataFrame:
        """
        Retrieves the salaries of NBA players for a specific season.

        Returns:
            pd.DataFrame: A DataFrame containing the salaries of NBA players for the specified season.
        """
        year = self.season.split("-")[0]
        season_string = year + "-" + str(int(year) + 1)

        url = f"https://hoopshype.com/salaries/players/{season_string}/"
        result = requests.get(url)
        soup = BeautifulSoup(result.content, features="html.parser")
        tables = soup.find_all("table")[0]
        
        # # Get the table rows
        rows = [[cell.text.strip() for cell in row.find_all('td')] for row in tables.find_all('tr')]

        self.salary_df = pd.DataFrame(rows[1:], columns=rows[0])
        if '' in self.salary_df.columns:
            self.salary_df = self.salary_df.drop(columns=[""])
            
        self.salary_df['Season'] = self.salary_df.columns[1].replace("/", "_")
        self.salary_df.columns = ["Player", "Salary", "Adj_Salary", "Season"]

        return self.salary_df

    def get_lineups(self):
        """
        Retrieves the lineups data for the specified season, season type, and per mode.

        Returns:
            pandas.DataFrame: The lineups data for the specified season, season type, and per mode.
        """
        self.lineups = nba.LeagueDashLineups(
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        return self.lineups

    def get_lineup_details(self):
        """
        Retrieves the lineup details for the specified season.

        Returns:
            pandas.DataFrame: The lineup details for the specified season.
        """
        self.lineup_details = nba.LeagueLineupViz(
            season=self.season,
            season_type_all_star=self.season_type,
            minutes_min=1,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        return self.lineup_details

    def get_opponent_shooting(self):
        """
        Retrieves the opponent shooting statistics for the specified season.

        Returns:
            pandas.DataFrame: The opponent shooting statistics for the season.
        """
        self.opponent_shooting = nba.LeagueDashOppPtShot(
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_simple=self.permode,
        ).get_data_frames()[0]
        return self.opponent_shooting

    def get_player_clutch(self):
        """
        Retrieves the player clutch data for the specified season.

        Returns:
            pandas.DataFrame: The player clutch data for the specified season.
        """
        self.player_clutch = nba.LeagueDashPlayerClutch(
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        return self.player_clutch

    def get_player_shots(self):
        """
        Retrieves the player shots data for the specified season, season type, and per mode.

        Returns:
            pandas.DataFrame: The player shots data.
        """
        self.player_shots = nba.LeagueDashPlayerPtShot(
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_simple=self.permode,
        ).get_data_frames()[0]
        return self.player_shots

    def get_player_shot_locations(self):
        """
        Retrieves the shot locations data for the players in the specified season.

        Returns:
            pandas.DataFrame: A DataFrame containing the shot locations data for the players.
        """
        self.player_shot_locations = nba.LeagueDashPlayerShotLocations(
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        return self.player_shot_locations

    def get_player_stats(self):
        """
        Retrieves the player statistics for the specified season.

        Returns:
            pandas.DataFrame: A DataFrame containing the player statistics.
        """
        self.player_stats = nba.LeagueDashPlayerStats(
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        return self.player_stats

    def get_team_clutch(self):
        """
        Retrieves the clutch statistics for teams in the specified season.

        Returns:
            pandas.DataFrame: A DataFrame containing the clutch statistics for teams.
        """
        self.team_clutch = nba.LeagueDashTeamClutch(
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        return self.team_clutch

    def get_team_shots_bypoint(self):
        """
        Retrieves the team shots by point data for the specified season.

        Returns:
            pandas.DataFrame: The team shots by point data.
        """
        self.team_shots_bypoint = nba.LeagueDashTeamPtShot(
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_simple=self.permode,
        ).get_data_frames()[0]
        return self.team_shots_bypoint

    def get_team_shot_locations(self):
        """
        Retrieves the shot locations data for teams in a specific season.

        Returns:
            pandas.DataFrame: The shot locations data for teams.
        """
        self.team_shot_locations = nba.LeagueDashTeamShotLocations(
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        return self.team_shot_locations

    def get_team_stats(self):
        """
        Retrieves the team statistics for the specified season.

        Returns:
            pandas.DataFrame: A DataFrame containing the team statistics.
        """
        self.team_stats = nba.LeagueDashTeamStats(
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        return self.team_stats

    def get_player_games(self) -> pd.DataFrame:
        """
        Retrieves the player games data for the specified season, season type, and per mode.

        Returns:
            pd.DataFrame: A DataFrame containing the player games data.
        """
        self.player_games = nba.PlayerGameLogs(
            season_nullable=self.season,
            season_type_nullable=self.season_type,
            per_mode_simple_nullable=self.permode,
        ).get_data_frames()[0]
        return self.player_games

    def get_team_games(self):
        """
        Retrieves the game log for a specific team in a given season.

        Returns:
            pandas.DataFrame: The game log data for the team.
        """
        self.team_games = nba.LeagueGameLog(
            season=self.season,
            season_type_all_star=self.season_type,
            player_or_team_abbreviation="T",
        ).get_data_frames()[0]
        return self.team_games

    def get_player_hustle(self):
        """
        Retrieves the hustle stats for players in the specified season and season type.

        Returns:
            pandas.DataFrame: A DataFrame containing the player hustle stats.
        """
        self.player_hustle = nba.LeagueHustleStatsPlayer(
            season=self.season,
            season_type_all_star=self.season_type,
        ).get_data_frames()[0]
        return self.player_hustle

    def get_team_hustle(self):
        """
        Retrieves the team hustle stats for the specified season and season type.

        Returns:
            pandas.DataFrame: The team hustle stats for the specified season and season type.
        """
        self.team_hustle = nba.LeagueHustleStatsTeam(
            season=self.season,
            season_type_all_star=self.season_type,
        ).get_data_frames()[0]
        return self.team_hustle

    def get_player_matchups(self):
        """
        Retrieves the player matchups for the current season.

        Returns:
            pandas.DataFrame: The player matchups data for the current season.
        """
        self.player_matchups = nba.LeagueSeasonMatchups(
            season=self.season,
            season_type_playoffs=self.season_type,
            per_mode_simple=self.permode,
        ).get_data_frames()[0]
        return self.player_matchups

    def get_player_estimated_metrics(self):
        """
        Retrieves the estimated metrics for players in the specified season and season type.

        Returns:
            pandas.DataFrame: A DataFrame containing the estimated metrics for players.
        """
        self.player_estimated_metrics = nba.PlayerEstimatedMetrics(
            season=self.season,
            season_type=self.season_type,
        ).get_data_frames()[0]
        return self.player_estimated_metrics

    def get_synergy_player(
        self, play_type: str = "Transition", offensive: bool = True
    ) -> pd.DataFrame:
        """
        Retrieves synergy data for a specific play type and offensive/defensive category.

        Args:
            play_type (str, optional): The play type to retrieve synergy data for. Defaults to "Transition".
            offensive (bool, optional): Specifies whether to retrieve offensive or defensive synergy data. Defaults to True.

        Returns:
            pd.DataFrame: The synergy data as a pandas DataFrame.
        """
        self.play_type = Formatter.check_playtype(
            play_type, playtypes=PlayTypes.PLAYTYPES
        )
        if offensive:
            self.off_def = "offensive"
        else:
            self.off_def = "defensive"

        if isinstance(self.play_type, str):
            self.synergy = nba.SynergyPlayTypes(
                season=self.season,
                per_mode_simple=self.permode,
                play_type_nullable=self.play_type,
                type_grouping_nullable=self.off_def,
                player_or_team_abbreviation="P",
                season_type_all_star=self.season_type,
            ).get_data_frames()[0]

        else:
            df_list = []
            for play in track(self.play_type):

                temp_df = nba.SynergyPlayTypes(
                    season=self.season,
                    per_mode_simple=self.permode,
                    play_type_nullable=play,
                    type_grouping_nullable=self.off_def,
                    player_or_team_abbreviation="P",
                    season_type_all_star=self.season_type,
                ).get_data_frames()[0]
                df_list.append(temp_df)
                sleep(1)

            self.synergy = pd.concat(df_list)

        return self.synergy

    def get_synergy_team(
        self, play_type: str = "Transition", offensive: bool = True
    ) -> pd.DataFrame:
        """
        Retrieves synergy data for a specific play type and team.

        Args:
            play_type (str, optional): The play type to retrieve synergy data for. Defaults to "Transition".
            offensive (bool, optional): Determines whether to retrieve offensive or defensive synergy data. Defaults to True.

        Returns:
            pd.DataFrame: A DataFrame containing the synergy data.

        Raises:
            ValueError: If the play type is not valid.

        """
        self.play_type = Formatter.check_playtype(
            play_type, playtypes=PlayTypes.PLAYTYPES
        )
        if offensive:
            self.off_def = "offensive"
        else:
            self.off_def = "defensive"

        if isinstance(self.play_type, str):
            self.synergy = nba.SynergyPlayTypes(
                season=self.season,
                per_mode_simple=self.permode,
                play_type_nullable=self.play_type,
                type_grouping_nullable=self.off_def,
                player_or_team_abbreviation="T",
                season_type_all_star=self.season_type,
            ).get_data_frames()[0]

        else:
            df_list = []
            for play in track(self.play_type):
                temp_df = nba.SynergyPlayTypes(
                    season=self.season,
                    per_mode_simple=self.permode,
                    play_type_nullable=play,
                    type_grouping_nullable=self.off_def,
                    player_or_team_abbreviation="T",
                    season_type_all_star=self.season_type,
                ).get_data_frames()[0]
                df_list.append(temp_df)
                sleep(1)

            self.synergy = pd.concat(df_list)

        return self.synergy

    def get_tracking_player(
        self,
        track_type: str = "Efficiency",
    ) -> pd.DataFrame:
        """
        Retrieves tracking data for players based on the specified track type.

        Parameters:
            track_type (str): The type of tracking data to retrieve. Defaults to "Efficiency".

        Returns:
            pd.DataFrame: A DataFrame containing the tracking data for players.
        """
        self.play_type = Formatter.check_playtype(
            track_type, playtypes=PlayTypes.TRACKING_TYPES
        )

        if isinstance(self.play_type, str):
            self.tracking = nba.LeagueDashPtStats(
                season=self.season,
                per_mode_simple=self.permode,
                pt_measure_type=self.play_type,
                player_or_team="Player",
                season_type_all_star=self.season_type,
            ).get_data_frames()[0]

        else:
            df_list = []
            for play in track(self.play_type):

                temp_df = nba.LeagueDashPtStats(
                    season=self.season,
                    per_mode_simple=self.permode,
                    pt_measure_type=play,
                    player_or_team="Player",
                    season_type_all_star=self.season_type,
                ).get_data_frames()[0]
                df_list.append(temp_df)
                sleep(1)

            self.tracking = pd.concat(df_list)

        return self.tracking

    def get_tracking_team(
        self,
        track_type: str = "Efficiency",
    ) -> pd.DataFrame:
        """
        Retrieves tracking data for a specific play type and returns it as a pandas DataFrame.

        Parameters:
            track_type (str): The play type to track. Defaults to "Efficiency".

        Returns:
            pd.DataFrame: The tracking data as a pandas DataFrame.
        """
        self.play_type = Formatter.check_playtype(track_type, PlayTypes.TRACKING_TYPES)

        if isinstance(self.play_type, str):
            self.tracking = nba.LeagueDashPtStats(
                season=self.season,
                per_mode_simple=self.permode,
                pt_measure_type=self.play_type,
                player_or_team="Team",
                season_type_all_star=self.season_type,
            ).get_data_frames()[0]

        else:
            df_list = []
            for play in track(self.play_type):

                temp_df = nba.LeagueDashPtStats(
                    season=self.season,
                    per_mode_simple=self.permode,
                    pt_measure_type=play,
                    player_or_team="Team",
                    season_type_all_star=self.season_type,
                ).get_data_frames()[0]
                df_list.append(temp_df)
                sleep(1)

            self.tracking = pd.concat(df_list)

        return self.tracking

    def get_defense_player(
        self,
        defense_type: str = "Overall",
    ) -> pd.DataFrame:
        """
        Retrieves the defensive player data based on the specified defense type.

        Args:
            defense_type (str, optional): The type of defense to retrieve. Defaults to "Overall".

        Returns:
            pd.DataFrame: The defensive player data.

        """
        self.play_type = Formatter.check_playtype(
            defense_type, playtypes=PlayTypes.DEFENSE_TYPES
        )

        if isinstance(self.play_type, str):
            self.defense = nba.LeagueDashPtDefend(
                season=self.season,
                per_mode_simple=self.permode,
                defense_category=self.play_type,
                season_type_all_star=self.season_type,
            ).get_data_frames()[0]

        else:
            df_list = []
            for play in track(self.play_type):

                temp_df = nba.LeagueDashPtDefend(
                    season=self.season,
                    per_mode_simple=self.permode,
                    defense_category=play,
                    season_type_all_star=self.season_type,
                ).get_data_frames()[0]
                df_list.append(temp_df)
                sleep(1)

            self.defense = pd.concat(df_list)

        return self.defense

    def get_defense_team(
        self,
        defense_type: str = "Overall",
    ) -> pd.DataFrame:
        """
        Retrieves the defensive statistics for teams based on the specified defense type.

        Args:
            defense_type (str, optional): The type of defense to retrieve statistics for. Defaults to "Overall".

        Returns:
            pd.DataFrame: A DataFrame containing the defensive statistics for teams.
        """
        self.play_type = Formatter.check_playtype(
            defense_type, playtypes=PlayTypes.DEFENSE_TYPES
        )

        if isinstance(self.play_type, str):
            self.defense = nba.LeagueDashPtTeamDefend(
                season=self.season,
                per_mode_simple=self.permode,
                defense_category=self.play_type,
                season_type_all_star=self.season_type,
            ).get_data_frames()[0]

        else:
            df_list = []
            for play in track(self.play_type):

                temp_df = nba.LeagueDashPtTeamDefend(
                    season=self.season,
                    per_mode_simple=self.permode,
                    defense_category=play,
                    season_type_all_star=self.season_type,
                ).get_data_frames()[0]
                df_list.append(temp_df)
                sleep(1)

            self.defense = pd.concat(df_list)

        return self.defense


if __name__ == "__main__":
    seas = Season(season_year="2004")
    print(seas.permode)
    print(seas.get_salaries())



from typing import List

import nba_api.stats.endpoints as nba
import pandas as pd
import requests
from bs4 import BeautifulSoup
from nba_api.stats.static import teams

from nbastatpy.utils import Formatter, PlayTypes


class Team:
    def __init__(
            self,
            team_abbreviation: str,
            season_year: str = None,
            playoffs=False,
            permode: str = "PerGame",
        ):
        """
        Initializes a Team object.

        Parameters:
        - team_abbreviation (str): The abbreviation of the NBA team.
        - season_year (str, optional): The season year. If not provided, the current season year will be used.
        - playoffs (bool, optional): Specifies whether the team's statistics are for playoffs. Default is False.
        - permode (str, optional): The mode for the team's statistics. Default is "PerGame".

        Attributes:
        - permode (str): The formatted permode for the team's statistics.
        - season_year (str): The season year.
        - info (dict): The information about the team.
        - season (str): The formatted season.
        - season_type (str): The type of season (Regular Season or Playoffs).
        """
        self.permode = PlayTypes.PERMODE[
            permode.replace("_", "").replace("-", "").upper()
        ]
        if season_year:
            self.season_year = season_year
        else:
            self.season_year = Formatter.get_current_season_year()

        self.info = teams.find_team_by_abbreviation(team_abbreviation)

        self.season = Formatter.format_season(self.season_year)
        self.season_type = "Regular Season"
        if playoffs:
            self.season_type = "Playoffs"

        for attr_name, value in self.info.items():
            setattr(self, attr_name.lower(), self.info.get(attr_name, None))

    def get_logo(self):
        """
        Retrieves and returns the logo of the NBA team in svg format.

        Returns:
            PIL.Image.Image: The logo image of the NBA team.
        """
        pic_url = f"https://cdn.nba.com/logos/nba/{self.id}/primary/L/logo.svg"
        pic = requests.get(pic_url)
        self.logo = pic.content
        return self.logo

    def get_roster(self) -> List[pd.DataFrame]:
        """
        Retrieves the roster of the team for the specified season.

        Returns:
            List[pd.DataFrame]: A list of pandas DataFrames containing the roster data.
        """
        self.roster = nba.CommonTeamRoster(
            self.id,
            season=self.season,
        ).get_data_frames()
        return self.roster

    def get_salary(self) -> pd.DataFrame:
        """
        Retrieves the salary information for the team from hoopshype.com.

        Returns:
            pandas.DataFrame: A DataFrame containing the salary information for the team.
        """
        tm_name = "_".join(self.full_name.split(" ")).lower()
        year = self.season.split("-")[0]
        season_string = year + "-" + str(int(year) + 1)
        self.salary_url = f"https://hoopshype.com/salaries/{tm_name}/{season_string}/"

        result = requests.get(self.salary_url)
        soup = BeautifulSoup(result.content, features="html.parser")
        tables = soup.find_all("table")
        
        rows = [[cell.text.strip() for cell in row.find_all('td')] for row in tables[0].find_all('tr')]
        
        if not rows[0]:
            rows.pop(0)
            if not rows:
                raise(ValueError(f"Season data unavailable for: {season_string}"))
        self.salary_df = pd.DataFrame(rows[1:], columns=rows[0])
        self.salary_df["Season"] = self.salary_df.columns[1].replace("/", "_")
        self.salary_df.columns = ["Player", "Salary", "Adjusted Salary", "Season"]

        return self.salary_df

    def get_year_by_year(self) -> pd.DataFrame:
        """
        Retrieves the year-by-year statistics for the team.

        Returns:
            pd.DataFrame: The year-by-year statistics for the team.
        """
        self.year_by_year = nba.TeamYearByYearStats(
            team_id=self.id, per_mode_simple=self.permode
        ).get_data_frames()[0]
        return self.year_by_year

    def get_general_splits(self) -> pd.DataFrame:
        """
        Retrieves the general splits data for the team.

        Returns:
            pd.DataFrame: The general splits data for the team.
        """
        drop_cols = [
            "TEAM_GAME_LOCATION",
            "GAME_RESULT",
            "SEASON_MONTH_NAME",
            "SEASON_SEGMENT",
            "TEAM_DAYS_REST_RANGE",
        ]
        self.general_splits = pd.concat(
            nba.TeamDashboardByGeneralSplits(
                team_id=self.id,
                season=self.season,
                season_type_all_star=self.season_type,
                per_mode_detailed=self.permode,
            ).get_data_frames()
        ).drop(columns=drop_cols)
        return self.general_splits

    def get_shooting_splits(self) -> pd.DataFrame:
        """
        Retrieves shooting splits data for the team.

        Returns:
            pd.DataFrame: The shooting splits data for the team.
        """
        self.shooting_splits = pd.concat(
            nba.TeamDashboardByShootingSplits(
                team_id=self.id,
                season=self.season,
                season_type_all_star=self.season_type,
                per_mode_detailed=self.permode,
            ).get_data_frames()
        )
        return self.shooting_splits

    def get_leaders(self) -> pd.DataFrame:
        """
        Retrieves the franchise leaders data for the team.

        Returns:
            pd.DataFrame: The franchise leaders data for the team.
        """
        self.leaders = nba.FranchiseLeaders(team_id=self.id).get_data_frames()[0]
        return self.leaders

    def get_franchise_players(self) -> pd.DataFrame:
        """
        Retrieves the franchise players for the team.

        Returns:
            pd.DataFrame: A DataFrame containing the franchise players' data.
        """
        self.franchise_players = nba.FranchisePlayers(
            team_id=self.id
        ).get_data_frames()[0]
        return self.franchise_players

    def get_season_lineups(self) -> pd.DataFrame:
        """
        Retrieves the season lineups for the team.

        Returns:
            pd.DataFrame: A DataFrame containing the season lineups data.
        """
        self.season_lineups = nba.LeagueDashLineups(
            team_id_nullable=self.id,
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        self.season_lineups["season"] = self.season
        self.season_lineups["season_type"] = self.season_type

        return self.season_lineups

    def get_opponent_shooting(self) -> pd.DataFrame:
        """
        Retrieves the opponent shooting statistics for the team.

        Returns:
            pd.DataFrame: DataFrame containing the opponent shooting statistics.
        """
        self.opponent_shooting = nba.LeagueDashOppPtShot(
            team_id_nullable=self.id,
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_simple=self.permode,
        ).get_data_frames()[0]
        self.opponent_shooting["season"] = self.season
        self.opponent_shooting["season_type"] = self.season_type

        return self.opponent_shooting

    def get_player_clutch(self) -> pd.DataFrame:
        """
        Retrieves the clutch statistics for the players of the team.

        Returns:
            pd.DataFrame: A DataFrame containing the clutch statistics for the players of the team.
        """
        self.player_clutch = nba.LeagueDashPlayerClutch(
            team_id_nullable=self.id,
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        self.player_clutch["season"] = self.season
        self.player_clutch["season_type"] = self.season_type

        return self.player_clutch

    def get_player_shots(self) -> pd.DataFrame:
        """
        Retrieves the player shots data for the team.

        Returns:
            pd.DataFrame: The player shots data for the team.
        """
        self.player_shots = nba.LeagueDashPlayerPtShot(
            team_id_nullable=self.id,
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_simple=self.permode,
        ).get_data_frames()[0]
        self.player_shots["season"] = self.season
        self.player_shots["season_type"] = self.season_type

        return self.player_shots

    def get_player_shot_locations(self) -> pd.DataFrame:
        """
        Retrieves the shot locations data for the players of the team.

        Returns:
            pd.DataFrame: A DataFrame containing the shot locations data for the players.
        """
        self.player_shot_locations = nba.LeagueDashPlayerShotLocations(
            team_id_nullable=self.id,
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        self.player_shot_locations["season"] = self.season
        self.player_shot_locations["season_type"] = self.season_type

        return self.player_shot_locations

    def get_player_stats(self) -> pd.DataFrame:
        """
        Retrieves the player statistics for the team.

        Returns:
            pd.DataFrame: A DataFrame containing the player statistics.
        """
        self.player_stats = nba.LeagueDashPlayerStats(
            team_id_nullable=self.id,
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        self.player_stats["season"] = self.season
        self.player_stats["season_type"] = self.season_type

        return self.player_stats

    def get_player_point_defend(self) -> pd.DataFrame:
        """
        Retrieves the player point defense data for the team.

        Returns:
            pd.DataFrame: The player point defense data for the team.
        """
        self.player_point_defend = nba.LeagueDashPtDefend(
            team_id_nullable=self.id,
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_simple=self.permode,
        ).get_data_frames()[0]
        self.player_point_defend["season"] = self.season
        self.player_point_defend["season_type"] = self.season_type

        return self.player_point_defend

    def get_player_hustle(self) -> pd.DataFrame:
        """
        Retrieves the hustle stats for the players of the team.

        Returns:
            pd.DataFrame: A DataFrame containing the hustle stats for the players.
        """
        self.player_hustle = nba.LeagueHustleStatsPlayer(
            team_id_nullable=self.id,
            season=self.season,
            season_type_all_star=self.season_type,
        ).get_data_frames()[0]
        self.player_hustle["season"] = self.season
        self.player_hustle["season_type"] = self.season_type

        return self.player_hustle

    def get_lineup_details(self) -> pd.DataFrame:
        """
        Retrieves the lineup details for the team.

        Returns:
            pd.DataFrame: The lineup details for the team.
        """
        self.lineup_details = nba.LeagueLineupViz(
            team_id_nullable=self.id,
            season=self.season,
            season_type_all_star=self.season_type,
            minutes_min=1,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        self.lineup_details["season"] = self.season
        self.lineup_details["season_type"] = self.season_type

        return self.lineup_details

    def get_player_on_details(self) -> pd.DataFrame:
        """
        Retrieves the player on-court details for the team.

        Returns:
            pd.DataFrame: A DataFrame containing the player on-court details.
        """
        self.player_on_details = nba.LeaguePlayerOnDetails(
            team_id=self.id,
            season=self.season,
            season_type_all_star=self.season_type,
            per_mode_detailed=self.permode,
        ).get_data_frames()[0]
        self.player_on_details["season"] = self.season
        self.player_on_details["season_type"] = self.season_type

        return self.player_on_details

    def get_player_matchups(self, defense=False) -> pd.DataFrame:
        """
        Retrieves player matchups for the team.

        Args:
            defense (bool, optional): If True, retrieves defensive matchups. If False, retrieves offensive matchups. Defaults to False.

        Returns:
            pd.DataFrame: DataFrame containing player matchups for the team.
        """
        if defense:
            self.player_matchups = nba.LeagueSeasonMatchups(
                def_team_id_nullable=self.id,
                season=self.season,
                season_type_playoffs=self.season_type,
                per_mode_simple=self.permode,
            ).get_data_frames()[0]
        else:
            self.player_matchups = nba.LeagueSeasonMatchups(
                off_team_id_nullable=self.id,
                season=self.season,
                season_type_playoffs=self.season_type,
                per_mode_simple=self.permode,
            ).get_data_frames()[0]

        self.player_matchups["season"] = self.season
        self.player_matchups["season_type"] = self.season_type

        return self.player_matchups

    def get_player_passes(self) -> pd.DataFrame:
        """
        Retrieves the player passes data for the team.

        Returns:
            pd.DataFrame: The player passes data for the team.
        """
        self.player_passes = pd.concat(
            nba.TeamDashPtPass(
                team_id=self.id,
                season=self.season,
                season_type_all_star=self.season_type,
                per_mode_simple=self.permode,
            ).get_data_frames()
        )

        group_cols = ["PASS_FROM", "PASS_TO"]
        self.player_passes["GROUP_SET"] = self.player_passes[group_cols].apply(
            Formatter.combine_strings, axis=1
        )
        self.player_passes = self.player_passes.drop(columns=group_cols)
        return self.player_passes.reset_index(drop=True)

    def get_player_onoff(self) -> pd.DataFrame:
        """
        Retrieves the on-off court details for the players of the team.

        Returns:
            pd.DataFrame: A DataFrame containing the on-off court details for the players.
        """
        self.player_onoff = pd.concat(
            nba.TeamPlayerOnOffDetails(
                team_id=team.id,
                season=self.season,
                season_type_all_star=self.season_type,
                per_mode_detailed=self.permode,
            ).get_data_frames()[1:]
        )
        return self.player_onoff.reset_index(drop=True)


if __name__ == "__main__":
    team_name = "MIL"
    team = Team(team_name, season_year="2024", playoffs=True)
    print(team.get_salary())



from typing import List

import nba_api.stats.endpoints as nba
import pandas as pd

from nbastatpy.utils import Formatter


class Game:
    def __init__(self, game_id: str):
        """This represents a game.  Given an ID, you can get boxscore (and other) information through one of the 'get' methods

        Args:
            game_id (str): string with 10 digits
        """
        self.game_id = Formatter.format_game_id(game_id)

    def get_boxscore(self) -> List[pd.DataFrame]:
        """Gets traditional boxscore

        Returns:
            List[pd.DataFrame]: list of dataframes (players, starters/bench, team)
        """
        self.boxscore = nba.BoxScoreTraditionalV3(self.game_id).get_data_frames()
        return self.boxscore

    def get_advanced(self):
        """
        Retrieves the advanced box score data for the game.

        Returns:
            pandas.DataFrame: The advanced box score data for the game.
        """
        self.adv_box = nba.BoxScoreAdvancedV3(self.game_id).get_data_frames()
        return self.adv_box

    def get_defense(self):
        """
        Retrieves the defensive statistics for the game.

        Returns:
            def_box (pandas.DataFrame): DataFrame containing the defensive statistics.
        """
        self.def_box = nba.BoxScoreDefensiveV2(self.game_id).get_data_frames()
        return self.def_box

    def get_four_factors(self):
        """
        Retrieves the four factors data for the game.

        Returns:
            pandas.DataFrame: The four factors data for the game.
        """
        self.four_factors = nba.BoxScoreFourFactorsV3(self.game_id).get_data_frames()
        return self.four_factors

    def get_hustle(self) -> List[pd.DataFrame]:
        """Gets hustle data for a given game

        Returns:
            List[pd.DataFrame]: list of two dataframes (players, teams)
        """
        self.hustle = nba.BoxScoreHustleV2(self.game_id).get_data_frames()
        return self.hustle

    def get_matchups(self):
        """
        Retrieves the matchups for the game.

        Returns:
            pandas.DataFrame: The matchups data for the game.
        """
        self.matchups = nba.BoxScoreMatchupsV3(self.game_id).get_data_frames()
        return self.matchups

    def get_misc(self):
        """
        Retrieves miscellaneous box score data for the game.

        Returns:
            pandas.DataFrame: The miscellaneous box score data.
        """
        self.misc = nba.BoxScoreMiscV3(self.game_id).get_data_frames()
        return self.misc

    def get_scoring(self):
        """
        Retrieves the scoring data for the game.

        Returns:
            pandas.DataFrame: The scoring data for the game.
        """
        self.scoring = nba.BoxScoreScoringV3(self.game_id).get_data_frames()
        return self.scoring

    def get_usage(self) -> List[pd.DataFrame]:
        """Gets usage data for a given game

        Returns:
            List[pd.DataFrame]: list of two dataframes (players, teams)
        """
        self.usage = nba.BoxScoreUsageV3(self.game_id).get_data_frames()
        return self.usage

    def get_playertrack(self):
        """
        Retrieves the player tracking data for the game.

        Returns:
            playertrack (pandas.DataFrame): The player tracking data for the game.
        """
        self.playertrack = nba.BoxScorePlayerTrackV3(self.game_id).get_data_frames()
        return self.playertrack

    def get_rotations(self):
        """
        Retrieves the rotations data for the game.

        Returns:
            pandas.DataFrame: The rotations data for the game.
        """
        self.rotations = pd.concat(
            nba.GameRotation(game_id=self.game_id).get_data_frames()
        )
        return self.rotations

    def get_playbyplay(self) -> pd.DataFrame:
        """
        Retrieves the play-by-play data for the game.

        Returns:
            pd.DataFrame: The play-by-play data as a pandas DataFrame.
        """
        self.playbyplay = nba.PlayByPlayV3(self.game_id).get_data_frames()[0]
        return self.playbyplay

    def get_win_probability(self) -> pd.DataFrame:
        """
        Retrieves the win probability data for the game.

        Returns:
            pd.DataFrame: The win probability data as a pandas DataFrame.
        """
        self.win_probability = nba.WinProbabilityPBP(
            game_id=self.game_id
        ).get_data_frames()[0]
        return self.win_probability


if __name__ == "__main__":
    GAME_ID = "0022301148"
    game = Game(game_id=GAME_ID)
    print(game.get_win_probability())
